@@ -4,8 +4,8 @@
 
   if (length(idx) > 0) {
 
-    LHS <- LHS[, -idx]
-    RHS <- RHS[, -idx]
+    LHS <- LHS[, -idx, drop = FALSE]
+    RHS <- RHS[, -idx, drop = FALSE]
 
   }
 
