@@ -1,6 +1,6 @@
 compute_grades <- function(I) {
 
-  lapply(seq(ncol(I)),
+  lapply(seq_len(ncol(I)),
          function(col) {
            v <- c(0, I[, col], 1)
            g <- sort(unique(v))
