@@ -6,14 +6,14 @@
   #  => remove C -> D
 
   RHS_subsets <- .subset(RHS)
-  LHS_subsets <- t(.subset(LHS))
+  LHS_subsets <- Matrix::t(.subset(LHS))
 
   ALL_subsets <- LHS_subsets & RHS_subsets
 
   # Find A subset of C
-  condition1 <- colSums(ALL_subsets) > 1
+  condition1 <- Matrix::colSums(ALL_subsets) > 1
 
-  subsets <- which(condition1)
+  subsets <- Matrix::which(condition1)
 
   marked_as_single <- rep(TRUE, ncol(LHS))
 
@@ -28,8 +28,8 @@
 
       # Select C -> D with A subset of C and B superset of D
       idx_subset <- which_at_col(ALL_subsets@i,
-                                   ALL_subsets@p,
-                                   this_row)
+                                 ALL_subsets@p,
+                                 this_row)
 
       idx_subset <- setdiff(idx_subset, this_row)
 
@@ -45,7 +45,7 @@
   LHS <- LHS[, singles]
   RHS <- RHS[, singles]
 
-  return(list(lhs = Matrix(LHS, sparse = TRUE),
-              rhs = Matrix(RHS, sparse = TRUE)))
+  return(list(lhs = Matrix::Matrix(LHS, sparse = TRUE),
+              rhs = Matrix::Matrix(RHS, sparse = TRUE)))
 
 }
