@@ -1,56 +1,32 @@
 #' @title
-#' R6 Class for Set of implications
+#' R6 class for an Implication Set
 #'
 #' @description
-#' This class implements the structure needed to store implications and the methods associated.
+#' This class implements an implication set (LHS -> RHS) in the framework of
+#' Formal Concept Analysis (FCA). It inherits from \code{RuleSet} and adds
+#' FCA-specific methods such as closure computation, simplification, and
+#' basis transformation.
 #'
-#' @examples
-#' # Build a formal context
-#' fc_planets <- FormalContext$new(planets)
-#'
-#' # Find its implication basis
-#' fc_planets$find_implications()
-#'
-#' # Print implications
-#' fc_planets$implications
-#'
-#' # Cardinality and mean size in the ruleset
-#' fc_planets$implications$cardinality()
-#' sizes <- fc_planets$implications$size()
-#' colMeans(sizes)
-#'
-#' # Simplify the implication set
-#' fc_planets$implications$apply_rules("simplification")
-#'
-#' @references
-#'
-#' Ganter B, Obiedkov S (2016). Conceptual Exploration. Springer. https://doi.org/10.1007/978-3-662-49291-8
-#'
-#' Hahsler M, Grun B, Hornik K (2005). “arules - a computational environment for mining association rules and frequent item sets.” _J Stat Softw_, *14*, 1-25.
-#'
-#' Belohlavek R, Cordero P, Enciso M, Mora Á, Vychodil V (2016). “Automated prover for attribute dependencies in data with grades.” _International Journal of Approximate Reasoning_, *70*, 51-67.
-#'
-#' Mora A, Cordero P, Enciso M, Fortes I, Aguilera G (2012). “Closure via functional dependence simplification.” _International Journal of Computer Mathematics_, *89*(4), 510-526.
+#' @importFrom Matrix colSums Matrix
+#' @importFrom stringr str_wrap
+#' @importFrom methods new
 #'
 #' @export
 ImplicationSet <- R6::R6Class(
-  classname = "ImplicationSet",
+  "ImplicationSet",
+  inherit = RuleSet,
+
   public = list(
     #' @description
-    #' Initialize with an optional name
+    #' Initialize an ImplicationSet
     #'
-    #' @param ... See Details.
-    #'
-    #' @details
-    #' Creates and initialize a new \code{ImplicationSet} object. It can be done in two ways:
-    #' \code{initialize(name, attributes, lhs, rhs)}
-    #' or \code{initialize(rules)}
-    #'
-    #' In the first way, the only mandatory argument is \code{attributes}, (character vector) which is a vector of names of the attributes on which we define the implications. Optional arguments are: \code{name} (character string), name of the implication set, \code{lhs} (a \code{dgCMatrix}), initial LHS of the implications stored and the analogous \code{rhs}.
-    #'
-    #' The other way is used to initialize the \code{ImplicationSet} object from a \code{rules} object from package \code{arules}.
+    #' @param ...   A \code{rules} object (from \code{arules}) or named arguments:
+    #' \code{name} (string), \code{attributes} (character vector),
+    #' \code{lhs} and \code{rhs} (sparse matrices), \code{I} (incidence matrix).
     #'
     #' @return A new \code{ImplicationSet} object.
+    #'
+    #' @export
     initialize = function(...) {
       dots <- list(...)
 
@@ -59,6 +35,7 @@ ImplicationSet <- R6::R6Class(
       any_rules <- which(classes == "rules")
 
       if (length(any_rules) > 0) {
+        check_needed_pkg("arules", "initialize from arules objects")
         arules_imp <- dots[[any_rules[1]]]
 
         attributes <- arules_imp@lhs@itemInfo$labels
@@ -72,6 +49,8 @@ ImplicationSet <- R6::R6Class(
 
         rownames(private$lhs_matrix) <- private$attributes
         rownames(private$rhs_matrix) <- private$attributes
+
+        private$quality <- data.frame()
       } else {
         args <- list(
           name = "",
@@ -86,95 +65,11 @@ ImplicationSet <- R6::R6Class(
         private$lhs_matrix <- args$lhs
         private$rhs_matrix <- args$rhs
         private$I <- args$I
+        private$quality <- data.frame()
         private$implication_support <- numeric(0)
       }
     },
 
-    #' @description
-    #' Get the names of the attributes
-    #'
-    #' @return A character vector with the names of the attributes used in the implications.
-    #'
-    #' @export
-    get_attributes = function() {
-      return(private$attributes)
-    },
-
-    #' @description
-    #' Get a subset of the implication set
-    #'
-    #' @param idx   (integer or logical vector) Indices of the implications to extract or remove. If logical vector, only \code{TRUE} elements are retained and the rest discarded.
-    #'
-    #' @return A new \code{ImplicationSet} with only the rules given by the \code{idx} indices (if all \code{idx > 0} and all but \code{idx} if all \code{idx < 0}.
-    #' @export
-    `[` = function(idx) {
-      if (is.logical(idx)) {
-        idx <- which(idx)
-      }
-
-      idx <- idx[abs(idx) <= self$cardinality()]
-      idx <- idx[abs(idx) > 0]
-
-      if (length(idx) > 0) {
-        if (all(idx < 0) | all(idx > 0)) {
-          imp <- ImplicationSet$new(
-            name = paste0(private$name, "_", paste0(idx)),
-            attributes = private$attributes,
-            lhs = Matrix::Matrix(private$lhs_matrix[, idx],
-              sparse = TRUE
-            ),
-            rhs = Matrix::Matrix(private$rhs_matrix[, idx],
-              sparse = TRUE
-            )
-          )
-
-          return(imp)
-        } else {
-          stop("Cannot use mixed positive and negative indices.\n",
-            call. = FALSE
-          )
-        }
-      } else {
-        return(ImplicationSet$new(
-          attributes = private$attributes,
-          I = private$I
-        ))
-      }
-    },
-
-    #' @description
-    #' Convert to arules format
-    #'
-    #' @param quality   (logical) Compute the interest measures for each rule?
-    #'
-    #' @return A \code{rules} object as used by package \code{arules}.
-    #'
-    #' @export
-    to_arules = function(quality = TRUE) {
-      if (self$is_empty()) {
-        stop("No implications to export.\n",
-          call. = FALSE
-        )
-      }
-
-      if (!private$is_binary()) {
-        stop("Export to arules format is only allowed when using binary implications.\n",
-          call. = FALSE
-        )
-      }
-
-      rules <- imps_to_arules(
-        LHS = private$lhs_matrix,
-        RHS = private$rhs_matrix,
-        attributes = private$attributes,
-        I = private$I,
-        quality = quality
-      )
-
-
-      return(rules)
-    },
-
     #' @description
     #' Add a precomputed implication set
     #'
@@ -184,16 +79,17 @@ ImplicationSet <- R6::R6Class(
     #'
     #' @export
     add = function(...) {
+      check_needed_pkg("arules", "import from arules objects")
       dots <- list(...)
 
       # Just a single ImplicationSet
       if (length(dots) == 1) {
         if (inherits(dots[[1]], "ImplicationSet")) {
-          private$append_implications(dots[[1]])
+          private$append_rules(dots[[1]])
         } else {
           # It is a rules object
           implications <- ImplicationSet$new(dots[[1]])
-          private$append_implications(implications)
+          private$append_rules(implications)
         }
       } else {
         # It must come in the form LHS, RHS
@@ -215,50 +111,112 @@ ImplicationSet <- R6::R6Class(
             rhs = rhs
           )
 
-          private$append_implications(imp)
+          private$append_rules(imp)
         } else {
-          stop("Invalid number of arguments.\n",
-            call. = FALSE
-          )
+          stop("Invalid number of arguments.\n", call. = FALSE)
         }
       }
     },
 
     #' @description
-    #' Cardinality: Number of implications in the set
+    #' Convert to arules format
     #'
-    #' @return The cardinality of the implication set.
+    #' @param quality   (logical) Compute the interest measures for each rule?
+    #'
+    #' @return A \code{rules} object as used by package \code{arules}.
     #'
     #' @export
-    cardinality = function() {
+    to_arules = function(quality = TRUE) {
+      check_needed_pkg("arules", "export to arules format")
       if (self$is_empty()) {
-        return(0)
+        stop("No implications to export.\n", call. = FALSE)
       }
 
-      ncol(private$lhs_matrix)
+      if (!private$is_binary()) {
+        stop(
+          "Export to arules format is only allowed when using binary implications.\n",
+          call. = FALSE
+        )
+      }
+
+      rules <- imps_to_arules(
+        LHS = private$lhs_matrix,
+        RHS = private$rhs_matrix,
+        attributes = private$attributes,
+        I = private$I,
+        quality = quality
+      )
+
+      return(rules)
     },
 
     #' @description
-    #' Empty set
+    #' Print all implications to text
     #'
-    #' @return \code{TRUE} if the set of implications is empty, \code{FALSE} otherwise.
+    #' @return A string with all the implications in the set.
     #'
     #' @export
-    is_empty = function() {
-      is.null(private$lhs_matrix)
+    print = function() {
+      if (is.null(private$lhs_matrix)) {
+        cat("Implication set with 0 implications.\n")
+
+        return(invisible(self))
+      }
+
+      n_implications <- ncol(private$lhs_matrix)
+      cat("Implication set with", n_implications, "implications.\n")
+
+      if (n_implications > 0) {
+        attributes <- private$attributes
+        LHS <- private$lhs_matrix
+        RHS <- private$rhs_matrix
+
+        implications <- sapply(
+          seq(n_implications),
+          function(i) {
+            paste0(
+              "Rule ",
+              i,
+              ": ",
+              .implication_to_string(LHS[, i], RHS[, i], attributes)
+            )
+          }
+        )
+
+        implications <- sapply(implications, function(s) {
+          stringr::str_wrap(s, width = getOption("width"), exdent = 2)
+        })
+
+        cat(implications, sep = "\n")
+      }
     },
 
     #' @description
-    #' Size: number of attributes in each of LHS and RHS
-    #'
-    #' @return A vector with two components: the number of attributes present in each of the LHS and RHS of each implication in the set.
+    #' Compute support of each implication
     #'
+    #' @return A vector with the support of each implication
     #' @export
-    size = function() {
-      lhs_size <- Matrix::colSums(private$lhs_matrix)
-      rhs_size <- Matrix::colSums(private$rhs_matrix)
+    support = function() {
+      if (self$is_empty()) {
+        return(numeric(0))
+      }
 
-      return(cbind(LHS = lhs_size, RHS = rhs_size))
+      if (length(private$implication_support) > 0) {
+        return(private$implication_support)
+      }
+
+      if (is.null(private$I)) {
+        return(numeric(0))
+      }
+
+      subsets <- .subset(
+        private$lhs_matrix,
+        private$I
+      )
+
+      private$implication_support <- Matrix::rowMeans(subsets)
+
+      return(private$implication_support)
     },
 
     #' @description
@@ -271,9 +229,7 @@ ImplicationSet <- R6::R6Class(
     #' @return If \code{reduce == FALSE}, the output is a fuzzy set corresponding to the closure of \code{S}. If \code{reduce == TRUE}, a list with two components: \code{closure}, with the closure as above, and \code{implications}, the reduced set of implications.
     #'
     #' @export
-    closure = function(S,
-                       reduce = FALSE,
-                       verbose = FALSE) {
+    closure = function(S, reduce = FALSE, verbose = FALSE) {
       if (inherits(S, "Set")) {
         original <- S$clone()
         S <- match_attributes(S, private$attributes)
@@ -293,7 +249,8 @@ ImplicationSet <- R6::R6Class(
           is_direct = private$directness,
           logic = private$logic
         )
-      } else { # Should be globalization
+      } else {
+        # Should be globalization
         cl <- .compute_closure(
           S,
           LHS = private$lhs_matrix,
@@ -306,10 +263,12 @@ ImplicationSet <- R6::R6Class(
       }
 
       if (!reduce) {
-        cl <- list(closure = Set$new(
-          attributes = private$attributes,
-          M = cl$closure
-        ))
+        cl <- list(
+          closure = Set$new(
+            attributes = private$attributes,
+            M = cl$closure
+          )
+        )
       } else {
         cl$closure <- Set$new(
           attributes = private$attributes,
@@ -370,10 +329,12 @@ ImplicationSet <- R6::R6Class(
     #' @return Nothing, just updates the internal matrices for LHS and RHS.
     #'
     #' @export
-    apply_rules = function(rules = c("composition", "generalization"),
-                           batch_size = 25000L,
-                           parallelize = FALSE,
-                           reorder = FALSE) {
+    apply_rules = function(
+      rules = c("composition", "generalization"),
+      batch_size = 25000L,
+      parallelize = FALSE,
+      reorder = FALSE
+    ) {
       # If no implications, do nothing
       if (is.null(private$lhs_matrix) || (ncol(private$lhs_matrix) == 0)) {
         return(invisible(self))
@@ -385,16 +346,13 @@ ImplicationSet <- R6::R6Class(
         rules <- intersect(rules, "reduction")
       }
 
-
       if (length(rules) > 0) {
-        L <- .batch_apply(
+        L <- .process_batch(
           LHS = private$lhs_matrix,
           RHS = private$rhs_matrix,
           attributes = private$attributes,
           rules = rules,
-          parallelize = parallelize,
-          batch_size = batch_size,
-          reorder = reorder
+          verbose = FALSE
         )
 
         private$lhs_matrix <- L$lhs
@@ -444,19 +402,22 @@ ImplicationSet <- R6::R6Class(
     #'
     #' @return Nothing, updates the \code{ImplicationSet} in place with the new basis.
     #' @export
-    to_direct_optimal = function(method = c("direct_optimal", "final_ts", "monotonic", "priority"),
-                                 verbose = FALSE) {
+    to_direct_optimal = function(
+      method = c(
+        "do_sp",
+        "direct_optimal",
+        "final_ts",
+        "monotonic",
+        "priority"
+      ),
+      verbose = FALSE
+    ) {
       method <- match.arg(method)
 
-      # TODO: What happens with different hedges??!!
-
       # 1. Prepare Inputs for C++
-      # Calculate L (set of truth degrees present in data + 0 + 1)
       vals <- unique(c(0, 1, private$lhs_matrix@x, private$rhs_matrix@x))
       vals <- sort(vals)
 
-      # Retrieve the logic name from the private field
-      # Note: Ensure private$logic holds one of: "Lukasiewicz", "Zadeh", "Godel", "Product"
       current_logic <- private$logic
 
       if (is.null(current_logic)) {
@@ -464,19 +425,52 @@ ImplicationSet <- R6::R6Class(
       }
 
       # 2. Call C++
-      # Now passing 'current_logic' as an argument
-      res_list <- switch(method,
+      res_list <- switch(
+        method,
+        "do_sp" = run_direct_optimal_sp_single_pass_rcpp_optimized(
+          private$lhs_matrix,
+          private$rhs_matrix,
+          private$attributes,
+          vals,
+          current_logic,
+          TRUE,
+          verbose
+        ),
         "direct_optimal" = run_direct_optimal_sp_rcpp_optimized(
-          private$lhs_matrix, private$rhs_matrix, private$attributes, vals, current_logic, TRUE, verbose
+          private$lhs_matrix,
+          private$rhs_matrix,
+          private$attributes,
+          vals,
+          current_logic,
+          TRUE,
+          verbose
         ),
         "final_ts" = run_final_ts_rcpp_optimized(
-          private$lhs_matrix, private$rhs_matrix, private$attributes, vals, current_logic, TRUE, verbose
+          private$lhs_matrix,
+          private$rhs_matrix,
+          private$attributes,
+          vals,
+          current_logic,
+          TRUE,
+          verbose
         ),
         "monotonic" = run_monotonic_incremental_rcpp_optimized(
-          private$lhs_matrix, private$rhs_matrix, private$attributes, vals, current_logic, TRUE, verbose
+          private$lhs_matrix,
+          private$rhs_matrix,
+          private$attributes,
+          vals,
+          current_logic,
+          TRUE,
+          verbose
         ),
         "priority" = run_priority_refinement_rcpp_optimized(
-          private$lhs_matrix, private$rhs_matrix, private$attributes, vals, current_logic, TRUE, verbose
+          private$lhs_matrix,
+          private$rhs_matrix,
+          private$attributes,
+          vals,
+          current_logic,
+          TRUE,
+          verbose
         )
       )
 
@@ -486,7 +480,6 @@ ImplicationSet <- R6::R6Class(
       n_attrs <- length(private$attributes)
 
       if (n_rules > 0) {
-        # Convert 0-based indices (C++) to 1-based indices (R)
         new_lhs <- Matrix::sparseMatrix(
           i = s_data$lhs_i + 1,
           j = s_data$lhs_j + 1,
@@ -513,293 +506,12 @@ ImplicationSet <- R6::R6Class(
       private$implication_support <- numeric(0)
       private$directness <- TRUE
 
-      if (verbose) {
-        cat("Algorithm finished using logic:", current_logic, "\n")
-        print(res_list$metrics)
-      }
+      cat("Algorithm finished using logic:", current_logic, "\n")
+      print(res_list$metrics)
 
       return(invisible(self))
     },
 
-    #' @description
-    #' Print all implications to text
-    #'
-    #' @return A string with all the implications in the set.
-    #'
-    #' @export
-    print = function() {
-      if (is.null(private$lhs_matrix)) {
-        cat("Implication set with 0 implications.\n")
-
-        return(invisible(self))
-      }
-
-      n_implications <- ncol(private$lhs_matrix)
-      cat("Implication set with", n_implications, "implications.\n")
-
-      if (n_implications > 0) {
-        attributes <- private$attributes
-        LHS <- private$lhs_matrix
-        RHS <- private$rhs_matrix
-
-        implications <- sapply(
-          seq(n_implications),
-          function(i) {
-            paste0(
-              "Rule ", i, ": ",
-              # function(i) paste0("Rule: ",
-              .implication_to_string(LHS[, i], RHS[, i], attributes)
-            )
-          }
-        )
-
-        implications <- sapply(implications, function(s) stringr::str_wrap(s, width = getOption("width"), exdent = 2))
-
-        cat(implications, sep = "\n")
-      }
-    },
-
-    #' @description
-    #' Export to LaTeX
-    #'
-    #' @param print (logical) Print to output?
-    #' @param ncols  (integer) Number of columns for the output.
-    #' @param numbered (logical) If \code{TRUE} (default), implications will be numbered in the output.
-    #' @param numbers (vector) If \code{numbered}, use these elements to enumerate the implications. The default is to enumerate 1, 2, ..., but can be changed.
-    #'
-    #' @return A string in LaTeX format that prints nicely all the implications.
-    #'
-    #' @export
-    to_latex = function(print = TRUE,
-                        ncols = 1,
-                        numbered = TRUE,
-                        numbers = seq(self$cardinality())) {
-      output <- imp_to_latex(self,
-        ncols = ncols,
-        numbered = numbered,
-        numbers = numbers
-      )
-
-      if (print) {
-        cat(output)
-      }
-
-      return(invisible(output))
-    },
-
-    #' @description
-    #' Get internal LHS matrix
-    #'
-    #' @return A sparse matrix representing the LHS of the implications in the set.
-    #'
-    #' @export
-    get_LHS_matrix = function() {
-      if (self$is_empty()) {
-        LHS <- Matrix::Matrix(FALSE,
-          nrow = length(private$attributes),
-          ncol = 1,
-          sparse = TRUE
-        )
-      } else {
-        LHS <- private$lhs_matrix
-      }
-
-      dimnames(LHS) <- list(
-        private$attributes,
-        paste0(seq(ncol(LHS)))
-      )
-
-      return(LHS)
-    },
-
-    #' @description
-    #' Get internal RHS matrix
-    #'
-    #' @return A sparse matrix representing the RHS of the implications in the set.
-    #'
-    #' @export
-    get_RHS_matrix = function() {
-      if (self$is_empty()) {
-        RHS <- Matrix::Matrix(FALSE,
-          nrow = length(private$attributes),
-          ncol = 1,
-          sparse = TRUE
-        )
-      } else {
-        RHS <- private$rhs_matrix
-      }
-
-      dimnames(RHS) <- list(
-        private$attributes,
-        paste0(seq(ncol(RHS)))
-      )
-
-
-      return(RHS)
-    },
-
-
-    #' @description
-    #' Filter implications by attributes in LHS and RHS
-    #'
-    #' @param lhs  (character vector) Names of the attributes to filter the LHS by. If \code{NULL}, no filtering is done on the LHS.
-    #' @param not_lhs  (character vector) Names of the attributes to not include in the LHS. If \code{NULL} (the default), it is not considered at all.
-    #' @param rhs  (character vector) Names of the attributes to filter the RHS by. If \code{NULL}, no filtering is done on the RHS.
-    #' @param not_rhs  (character vector) Names of the attributes to not include in the RHS. If \code{NULL} (the default), it is not considered at all.
-    #' @param drop  (logical) Remove the rest of attributes in RHS?
-    #'
-    #' @return An \code{ImplicationSet} that is a subset of the current set, only with those rules which has the attributes in \code{lhs} and \code{rhs} in their LHS and RHS, respectively.
-    #'
-    #' @export
-    filter = function(lhs = NULL,
-                      not_lhs = NULL,
-                      rhs = NULL,
-                      not_rhs = NULL,
-                      drop = FALSE) {
-      RHS <- private$rhs_matrix
-      LHS <- private$lhs_matrix
-
-      if (!is.null(lhs)) {
-        # Filter the implications which have
-        # the given lhs
-        idx_attr <- match(
-          lhs,
-          private$attributes
-        )
-
-        if (length(idx_attr) > 1) {
-          idx_lhs <- Matrix::which(Matrix::colSums(LHS[idx_attr, ]) > 0)
-        } else {
-          idx_lhs <- which(LHS[idx_attr, ] > 0)
-        }
-      } else {
-        # If not specified a filter for LHS,
-        # select all implications
-        idx_lhs <- seq(ncol(LHS))
-      }
-
-      if (!is.null(not_lhs)) {
-        # Filter the implications which
-        # does not have the given not_lhs
-        idx_attr <- match(
-          not_lhs,
-          private$attributes
-        )
-
-        if (length(idx_attr) > 1) {
-          idx_not_lhs <- Matrix::which(Matrix::colSums(LHS[idx_attr, ]) > 0)
-        } else {
-          idx_not_lhs <- which(LHS[idx_attr, ] > 0)
-        }
-      } else {
-        # If not specified a filter for LHS,
-        # select none.
-        idx_not_lhs <- c()
-      }
-
-      idx_lhs <- setdiff(idx_lhs, idx_not_lhs)
-
-      if (!is.null(rhs)) {
-        # Filter the implications which have
-        # the given lhs
-        idx_attr <- match(
-          rhs,
-          private$attributes
-        )
-
-        if (length(idx_attr) > 1) {
-          idx_rhs <- Matrix::which(Matrix::colSums(RHS[idx_attr, ]) > 0)
-        } else {
-          idx_rhs <- which(RHS[idx_attr, ] > 0)
-        }
-      } else {
-        # If not specified a filter for RHS,
-        # select all implications
-        idx_rhs <- seq(ncol(RHS))
-      }
-
-      if (!is.null(not_rhs)) {
-        # Filter the implications which
-        # does not have the given not_lhs
-        idx_attr <- match(
-          not_rhs,
-          private$attributes
-        )
-
-        if (length(idx_attr) > 1) {
-          idx_not_rhs <- Matrix::which(Matrix::colSums(RHS[idx_attr, ]) > 0)
-        } else {
-          idx_not_rhs <- which(RHS[idx_attr, ] > 0)
-        }
-      } else {
-        # If not specified a filter for LHS,
-        # select none.
-        idx_not_rhs <- c()
-      }
-
-      idx_rhs <- setdiff(idx_rhs, idx_not_rhs)
-
-      idx <- intersect(idx_lhs, idx_rhs)
-
-      if (length(idx) == 0) {
-        warning("No combination of given LHS and RHS found.\n",
-          call. = FALSE,
-          immediate. = TRUE
-        )
-        return(invisible(NULL))
-      }
-
-      if (length(idx) > 0) {
-        if (drop && !is.null(rhs)) {
-          newLHS <- LHS[, idx]
-          newRHS <- RHS[, idx]
-
-          other_idx <- setdiff(seq(nrow(RHS)), idx_attr)
-          newRHS[other_idx, ] <- 0
-
-          imp <- ImplicationSet$new(
-            name = paste0(private$name, "_filtered"),
-            attributes = private$attributes,
-            lhs = Matrix::Matrix(newLHS, sparse = TRUE),
-            rhs = Matrix::Matrix(newRHS, sparse = TRUE)
-          )
-        } else {
-          imp <- ImplicationSet$new(
-            name = paste0(private$name, "_filtered"),
-            attributes = private$attributes,
-            lhs = Matrix::Matrix(LHS[, idx], sparse = TRUE),
-            rhs = Matrix::Matrix(RHS[, idx], sparse = TRUE)
-          )
-        }
-
-        return(imp)
-      }
-    },
-
-    #' @description
-    #' Compute support of each implication
-    #'
-    #' @return A vector with the support of each implication
-    #' @export
-    support = function() {
-      if (self$is_empty()) {
-        return(numeric(0))
-      }
-
-      if (length(private$implication_support) > 0) {
-        return(private$implication_support)
-      }
-
-      subsets <- .subset(
-        private$lhs_matrix,
-        private$I
-      )
-
-      private$implication_support <- Matrix::rowMeans(subsets)
-
-      return(private$implication_support)
-    },
-
     #' @description
     #' Sets the logic to use
     #'
@@ -848,19 +560,115 @@ ImplicationSet <- R6::R6Class(
     #' @export
     get_hedge = function() {
       private$hedge
+    },
+
+    #' @description
+    #' Export the implication set to JSON
+    #'
+    #' @param file        (character) The path of the file to save the JSON to.
+    #' @param return_list (logical) If TRUE, returns the list representation instead of the JSON string.
+    #'
+    #' @return A JSON string representing the implication set, or a list if \code{return_list} is TRUE.
+    #' @export
+    to_json = function(file = NULL, return_list = FALSE) {
+      check_needed_pkg("jsonlite", "export to JSON")
+
+      if (self$is_empty()) {
+        out <- list(
+          type = private$json_type_name(),
+          attributes = private$attributes,
+          rules = list()
+        )
+        if (return_list) {
+          return(out)
+        }
+        return(jsonlite::toJSON(out, auto_unbox = TRUE))
+      }
+
+      # Determine if fuzzy or binary
+      is_bin <- private$is_binary()
+
+      mat_to_list <- function(M, attrs) {
+        if (is.null(M)) {
+          return(replicate(length(attrs), list()))
+        }
+
+        # Force conversion to general triplet
+        T <- as(M, "TsparseMatrix")
+
+        if (length(T@i) == 0) {
+          return(replicate(ncol(M), list()))
+        }
+
+        # Build data frame with 1-based indices
+        df_all <- data.frame(
+          attr = attrs[T@i + 1],
+          rule = T@j + 1
+        )
+
+        res <- split(
+          df_all$attr,
+          factor(df_all$rule, levels = seq_len(ncol(M)))
+        )
+        return(res)
+      }
+
+      lhs_list <- mat_to_list(private$lhs_matrix, private$attributes)
+      rhs_list <- mat_to_list(private$rhs_matrix, private$attributes)
+
+      n_imps <- self$cardinality()
+
+      rules <- lapply(seq_len(n_imps), function(i) {
+        r <- list(
+          lhs = lhs_list[[i]],
+          rhs = rhs_list[[i]]
+        )
+        return(r)
+      })
+
+      if (
+        !is.null(private$implication_support) &&
+          length(private$implication_support) > 0
+      ) {
+        for (i in seq_along(rules)) {
+          rules[[i]]$support <- private$implication_support[i]
+        }
+      }
+
+      out <- list(
+        type = private$json_type_name(),
+        attributes = private$attributes,
+        rules = rules
+      )
+
+      if (return_list) {
+        return(out)
+      }
+
+      json <- jsonlite::toJSON(out, auto_unbox = TRUE)
+
+      if (!is.null(file)) {
+        writeLines(json, file)
+        return(invisible(json))
+      } else {
+        return(json)
+      }
     }
   ),
+
   private = list(
-    name = "",
-    attributes = NULL,
-    lhs_matrix = NULL,
-    rhs_matrix = NULL,
-    I = NULL,
+    # ImplicationSet-specific fields
     implication_support = NULL,
     binary = NULL,
     directness = FALSE,
     logic = "Godel",
     hedge = "globalization",
+
+    # Override parent helpers
+    rule_type_label = function() "Implication",
+    json_type_name = function() "ImplicationSet",
+
+    # Override is_binary with caching version
     is_binary = function() {
       if (!is.null(private$binary)) {
         return(private$binary)
@@ -871,8 +679,10 @@ ImplicationSet <- R6::R6Class(
       } else {
         if (!is.null(private$lhs_matrix)) {
           v <- unique(c(
-            0, private$lhs_matrix@x,
-            private$rhs_matrix@x, 1
+            0,
+            private$lhs_matrix@x,
+            private$rhs_matrix@x,
+            1
           ))
         } else {
           return(FALSE)
@@ -882,7 +692,14 @@ ImplicationSet <- R6::R6Class(
       private$binary <- (length(v) == 2) && all(v == c(0, 1))
       return(private$binary)
     },
-    append_implications = function(implications) {
+
+    # Override factory method
+    create_subset = function(...) {
+      ImplicationSet$new(...)
+    },
+
+    # Override append to match attributes
+    append_rules = function(implications) {
       imps <- match_implications(
         implications,
         private$attributes
@@ -900,3 +717,90 @@ ImplicationSet <- R6::R6Class(
     }
   )
 )
+#' @title Import ImplicationSet from JSON
+#' @description Reconstructs an ImplicationSet object from a JSON string.
+#' @param json_str A JSON string generated by \code{to_json()}.
+#' @return An \code{ImplicationSet} object.
+#' @export
+implications_from_json <- function(json_str) {
+  check_needed_pkg("jsonlite", "import from JSON")
+
+  data <- jsonlite::fromJSON(json_str, simplifyVector = FALSE)
+
+  if (data$type != "ImplicationSet") {
+    stop("Invalid JSON: type must be 'ImplicationSet'")
+  }
+
+  attributes <- unlist(data$attributes)
+  n_rules <- length(data$rules)
+
+  if (n_rules == 0) {
+    return(ImplicationSet$new(attributes = attributes))
+  }
+
+  # Reconstruct matrices
+  # We need to parse LHS and RHS from the rules list
+
+  # Helper to parse list of attributes (binary) or named list (fuzzy)
+  parse_part <- function(part_list) {
+    # We build a triplet matrix
+    i <- integer(0) # Attribute indices
+    j <- integer(0) # Rule indices
+    x <- numeric(0) # Values
+
+    for (rule_idx in seq_along(part_list)) {
+      item <- part_list[[rule_idx]]
+      if (length(item) > 0) {
+        # Check if it has names (fuzzy) or is just a list of strings (binary)
+        if (!is.null(names(item))) {
+          # Fuzzy: item is list(attr1 = val1, attr2 = val2)
+          attrs <- names(item)
+          vals <- unlist(item)
+        } else {
+          # Binary: item is list("attr1", "attr2") or vector
+          attrs <- unlist(item)
+          vals <- rep(1, length(attrs))
+        }
+
+        # Match attributes to indices
+        attr_idxs <- match(attrs, attributes)
+
+        if (any(is.na(attr_idxs))) {
+          warning(sprintf(
+            "Unknown attributes in rule %d: %s",
+            rule_idx,
+            paste(attrs[is.na(attr_idxs)], collapse = ", ")
+          ))
+          # Filter valid ones
+          valid <- !is.na(attr_idxs)
+          attr_idxs <- attr_idxs[valid]
+          vals <- vals[valid]
+        }
+
+        if (length(attr_idxs) > 0) {
+          i <- c(i, attr_idxs)
+          j <- c(j, rep(rule_idx, length(attr_idxs)))
+          x <- c(x, vals)
+        }
+      }
+    }
+
+    Matrix::sparseMatrix(
+      i = i,
+      j = j,
+      x = x,
+      dims = c(length(attributes), n_rules),
+      dimnames = list(attributes, NULL)
+    )
+  }
+
+  lhs_data <- lapply(data$rules, function(r) r$lhs)
+  rhs_data <- lapply(data$rules, function(r) r$rhs)
+
+  LHS <- parse_part(lhs_data)
+  RHS <- parse_part(rhs_data)
+
+  IS <- ImplicationSet$new(attributes = attributes, lhs = LHS, rhs = RHS)
+
+  return(IS)
+}
